<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */

include_once './Services/Export/classes/class.ilXmlExporter.php';
include_once './Services/Xml/classes/class.ilXmlWriter.php';
include_once './Modules/TestQuestionPool/classes/class.ilObjQuestionPool.php';

/**
 * Class ilQuestionSetPoolExporter
 * author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilQuestionSetPoolExporter extends ilXmlExporter
{
	/**
	 * @var ilXmlWriter
	 */
	protected $xml_writer = NULL;
	
	/**
	 * @var string
	 */
	protected $export_dir = '';
	
	/**
	 * @var string
	 */
	protected $subdir = '';
	
	/**
	 * @var string
	 */
	protected $filename = '';
	
	/**
	 * @var string
	 */
	protected $qti_filename = '';
	
	/**
	 * @var string
	 */
	protected $zipfilename = '';
	
	/**
	 * @var array with question ids to export
	 */
	protected $questions = array();
	
	/**
	 * @var ilLog
	 */
	protected $expLog = NULL;
	
	/**
	 * Initialisation
	 */
	function init()
	{
	}
	
	/**
	 * ilQuestionSetPoolExporter constructor.
	 */
	public function __construct()
	{
		$this->plugin_object = ilPlugin::getPluginObject('Services', 'Repository', 'robj', 'QuestionSetPool');
	}
	
	/**
	 *
	 */
	public function initLogFile()
	{
		include_once "./Services/Logging/classes/class.ilLog.php";
		$this->expLog = new ilLog($this->getExportDirectory(), "export.log");
		$this->expLog->delete();
		$this->expLog->setLogFormat("");
		$this->expLog->write(date("[y-m-d H:i:s] ") . "Start Export");
	}
	
	/**
	 * creates data directory for export files
	 * (data_dir/qpl_data/qpl_<id>/export, depending on data
	 * directory that is set in ILIAS setup/ini)
	 */
	function createExportDirectory()
	{
		global $ilErr;
		
		include_once "./Services/Utilities/classes/class.ilUtil.php";
		$xqsp_data_dir = ilUtil::getDataDir() . "/xqsp_data";
		ilUtil::makeDir($xqsp_data_dir);
		if(!is_writable($xqsp_data_dir))
		{
			$ilErr->raiseError("Questionpool Data Directory (" . $xqsp_data_dir . ") not writeable.", $ilErr->FATAL);
		}
		
		// create learning module directory (data_dir/lm_data/lm_<id>)
		$xqsp_dir = $xqsp_data_dir . "/xqsp_" . $this->object->getId();
		ilUtil::makeDir($xqsp_dir);
		if(!@is_dir($xqsp_dir))
		{
			$ilErr->raiseError("Creation of QuestionSetPool Directory failed. xqsp_", $ilErr->FATAL);
		}
		//		// create Export subdirectory (data_dir/lm_data/lm_<id>/Export)
		//		ilUtil::makeDir($this->getExportDirectory('xls'));
		//		if(!@is_dir($this->getExportDirectory('xls')))
		//		{
		//			$ilErr->raiseError("Creation of Export Directory failed. xls", $ilErr->FATAL);
		//		}
		ilUtil::makeDir($this->getExportDirectory('zip'));
		if(!@is_dir($this->getExportDirectory('zip')))
		{
			$ilErr->raiseError("Creation of Export Directory failed. zip", $ilErr->FATAL);
		}
		
	}
	
	/**
	 * get export directory of questionsetpool
	 * @param string $type
	 * @return string
	 */
	function getExportDirectory($type = "")
	{
		include_once "./Services/Utilities/classes/class.ilUtil.php";
		switch($type)
		{
			case 'xml':
				include_once("./Services/Export/classes/class.ilExport.php");
				$export_dir = ilExport::_getExportDirectory($this->object->getId(), $type, $this->object->getType());
				break;
//			case 'xls':
			case 'zip':
				$export_dir = ilUtil::getDataDir()."/xqsp_data"."/xqsp_".$this->object->getId()."/export_$type";
				break;
		
			default:
				$export_dir = ilUtil::getDataDir()."/xqsp_data"."/xqsp_".$this->object->getId()."/export";
				break;
		}
		return $export_dir;
	}

	/**
	 *
	 */
	public function exportPagesXML()
	{
		global $ilBench;
		
		$this->mob_ids  = array();
		$this->file_ids = array();
		
		$attrs         = array();
		$attrs["Type"] = "QuestionSetPool_FormATest";
		$this->xml_writer->xmlStartTag("ContentObject", $attrs);
		
		// MetaData
		$this->exportXMLMetaData();
		
		// Settings
		$this->exportXMLSettings();
		
		// Question Sets
		$this->exportQuestionSets();
		
		// PageObjects
		$this->expLog->write(date("[y-m-d H:i:s] ") . "Start Export Page Objects");
		$ilBench->start("ContentObjectExport", "exportPageObjects");
		$this->qpl_obj->exportXMLPageObjects($this->xml_writer, IL_INST_ID, $this->expLog, $this->questions);
		$ilBench->stop("ContentObjectExport", "exportPageObjects");
		$this->expLog->write(date("[y-m-d H:i:s] ") . "Finished Export Page Objects");
		
		// add media objects which were added with tiny mce
		$this->expLog->write(date("[y-m-d H:i:s] ") . "Start Export Media Objects");
		$export_dir = $this->exp->export_run_dir;
		$ilBench->start("ContentObjectExport", "exportMediaObjects");
		
		$this->exportXHTMLMediaObjects($export_dir);
		$ilBench->stop("ContentObjectExport", "exportMediaObjects");
		$this->expLog->write(date("[y-m-d H:i:s] ") . "Finished Export Media Objects");
		
		if(is_array($this->qpl_obj->file_ids) && count($this->qpl_obj->file_ids) > 0)
		{
			// FileItems
			$this->expLog->write(date("[y-m-d H:i:s] ") . "Start Export File Items");
			$ilBench->start("ContentObjectExport", "exportFileItems");
			$this->exportFileItems($this->export_dir . "/" . $this->subdir, $this->expLog);
			$ilBench->stop("ContentObjectExport", "exportFileItems");
			$this->expLog->write(date("[y-m-d H:i:s] ") . "Finished Export File Items");
		}
		$this->xml_writer->xmlEndTag("ContentObject");
	}
	
	/**
	 * export content objects meta data to xml (see ilias_co.dtd)
	 */
	function exportXMLMetaData()
	{
		include_once("Services/MetaData/classes/class.ilMD2XML.php");
		$md2xml = new ilMD2XML($this->object->getId(), 0, $this->object->getType());
		$md2xml->setExportMode(true);
		$md2xml->startExport();
		$this->xml_writer->appendXML($md2xml->getXML());
	}
	
	/**
	 *
	 */
	private function exportXMLSettings()
	{
		$this->xml_writer->xmlStartTag('Settings');
		
		$this->xml_writer->xmlElement('Title', null, (string)$this->object->getTitle());
		$this->xml_writer->xmlElement('Description', null, (string)$this->object->getDescription());
		$this->xml_writer->xmlElement('Online', null, (int)$this->object->isOnline());
		$this->xml_writer->xmlElement('SkillService', null, (int)$this->object->isSkillServiceEnabled());
		
		$this->xml_writer->xmlEndTag('Settings');
	}
	
	/**
	 *
	 */
	protected function exportQuestionSets()
	{
		$this->plugin_object = ilQuestionSetPoolPlugin::getInstance();
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolSet.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolPath.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolNode.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolJumpCondition.php');
		
		$setlist = $this->object->getSetList();
		$sets    = array();
		$sets    = $setlist->loadData();
		
		$this->xml_writer->xmlStartTag('QuestionSets');
		foreach($sets as $set_index => $set_data)
		{
			$set = new ilQuestionSetPoolSet((int)$set_data['id']);
			$set->read();
			
			$this->xml_writer->xmlStartTag('Set');
			$this->xml_writer->xmlElement('SetId', null, (int)$set->getId());
			$this->xml_writer->xmlElement('SetTitle', null, (string)$set->getTitle());
			$this->xml_writer->xmlElement('SetDescription', null, (string)$set->getDescription());
			
			$this->xml_writer->xmlElement('OriginalId', null, (int)$set->getOriginalId());
			
			// Export QuestionList 
			$this->exportQuestionList($set->getQuestionList());
			
			// Export Paths
			$this->exportPathList($set->getPathList());
			
			$this->xml_writer->xmlEndTag('Set');
		}
		
		$this->xml_writer->xmlEndTag('QuestionSets');
	}
	
	/**
	 * @param ilQuestionSetPoolSetQuestionList $questionList
	 */
	protected function exportQuestionList(ilQuestionSetPoolSetQuestionList $questionList)
	{
		$questions = array();
		$questions = $questionList->loadData();
		$this->xml_writer->xmlStartTag('Questions');
		foreach($questions as $question_index => $question)
		{
			$this->xml_writer->xmlStartTag('Question');
			$this->xml_writer->xmlElement('QuestionId', null, $question['question_id']);
			$this->xml_writer->xmlElement('QuestionIndex', null, $question['question_index']);
			$this->xml_writer->xmlElement('QuestionOriginalId', null, $question['original_id']);
			$this->xml_writer->xmlEndTag('Question');
			$this->questions[$question['question_id']] = $question['question_id'];
		}
		$this->xml_writer->xmlEndTag('Questions');
		if(count($this->questions) > 0)
		{
			$this->exportQuestions();
		}
	}
	
	/**
	 * @param ilQuestionSetPoolPathList $pathlist
	 */
	protected function exportPathList(ilQuestionSetPoolPathList $pathList)
	{
		$paths = array();
		$paths = $pathList->loadData();
		
		$this->xml_writer->xmlStartTag('Paths');
		foreach($paths as $path_index => $path_data)
		{
			$path = new ilQuestionSetPoolPath();
			$path->setId($path_data['id']);
			$path->read();
			$this->xml_writer->xmlStartTag('Path');
			$this->xml_writer->xmlElement('PathId', null, (int)$path->getId());
			$this->xml_writer->xmlElement('PathTitle', null, (string)$path->getTitle());
			$this->xml_writer->xmlElement('PathQuestionSetId', null, (int)$path_data['questionset_fi']);
			$this->xml_writer->xmlElement('PathIndex', null, $path->getIndex());
			
			// Nodes 
			$this->exportNodeList($path->getNodes());
			
			$this->xml_writer->xmlEndTag('Path');
		}
		$this->xml_writer->xmlEndTag('Paths');
	}
	
	/**
	 * @param ilQuestionSetPoolNodeList $nodelist
	 */
	protected function exportNodeList(ilQuestionSetPoolNodeList $nodeList)
	{
		$nodes = array();
		$nodes = $nodeList->loadData();
		
		$this->xml_writer->xmlStartTag('Nodes');
		foreach($nodes as $node_index => $node_data)
		{
			$node = new ilQuestionSetPoolNode((int)$node_data['id']);
			$node->read();
			$this->xml_writer->xmlStartTag('Node');
			
			$this->xml_writer->xmlElement('NodeId', null, (int)$node->getId());
			$this->xml_writer->xmlElement('NodeQuestionId', null, (int)$node_data['question_fi']);
			$this->xml_writer->xmlElement('NodePathId', null, (int)$node_data['path_fi']);
			$this->xml_writer->xmlElement('NodeIndex', null, (int)$node->getIndex());
			
			$this->xml_writer->xmlElement('NodeInitialCondition', null, $node->getInitialCondition());
			$this->xml_writer->xmlElement('NodeDominantScoring', null, $node->getDominantScoring());
			
			$this->xml_writer->xmlElement('NodeTrueFeedback', null, $node->getTrueFeedback());
			$this->xml_writer->xmlElement('NodeFalseFeedback', null, $node->getFalseFeedback());
			
			// Jump Conditions 
			$this->exportJumpConditions($node->getJumpConditionList());
			
			$this->xml_writer->xmlEndTag('Node');
		}
		$this->xml_writer->xmlEndTag('Nodes');
	}
	
	/**
	 * @param ilQuestionSetPoolJumpConditionList $jumpConditionList
	 */
	protected function exportJumpConditions(ilQuestionSetPoolJumpConditionList $jumpConditionList)
	{
		$jumpConditions = array();
		$jumpConditions = $jumpConditionList->getData();
		
		$this->xml_writer->xmlStartTag('JumpConditions');
		
		foreach($jumpConditions as $jmp_con_index => $jumpCondition_data)
		{
			$jumpCondition = new ilQuestionSetPoolJumpCondition();
			$jumpCondition->setId($jumpCondition_data['id']);
			$jumpCondition->read();
			
			$this->xml_writer->xmlStartTag('JumpCondition');
			$this->xml_writer->xmlElement('JumpConditionId', null, (int)$jumpCondition->getId());
			$this->xml_writer->xmlElement('JumpConditionText', null, (string)$jumpCondition->getJumpCondition());
			
			$this->xml_writer->xmlElement('JumpConditionNodeId', null, (int)$jumpCondition_data['node_fi']);
			$this->xml_writer->xmlElement('JumpConditionQuestionId', null, $jumpCondition_data['question_fi']);
			
			$this->xml_writer->xmlElement('JumpConditionRepetitions', null, $jumpCondition->getRepititions());
			$this->xml_writer->xmlElement('JumpConditionSorting', null, $jumpCondition->getSorting());
			$this->xml_writer->xmlElement('JumpConditionTrueFeedback', null, $jumpCondition->getTrueFeedback());
			$this->xml_writer->xmlElement('JumpConditionTrueJumpFeedback', null, $jumpCondition->getTrueJumpFeedback());
			$this->xml_writer->xmlElement('JumpConditionFalseFeedback', null, $jumpCondition->getFalseFeedback());
			
			$this->xml_writer->xmlEndTag('JumpCondition');
		}
		
		$this->xml_writer->xmlEndTag('JumpConditions');
	}
	
	/**
	 * Default ilias questionpool export
	 */
	protected function exportQuestions()
	{
		// write qti file
		ilUtil::makeDirParents($this->export_dir);
		$qti_file = fopen($this->export_dir . "/" . $this->qti_filename, "w");
		fwrite($qti_file, $this->qpl_obj->questionsToXML($this->questions));
		fclose($qti_file);
	}
	
	/**
	 * @param $a_export_dir
	 */
	function exportXHTMLMediaObjects($a_export_dir)
	{
		include_once("./Services/MediaObjects/classes/class.ilObjMediaObject.php");
		ilUtil::makeDirParents($a_export_dir."/objects/");
		
		foreach($this->questions as $question_id)
		{
			$mobs = ilObjMediaObject::_getMobsOfObject("qpl:html", $question_id);
			foreach($mobs as $mob)
			{
				if(ilObjMediaObject::_exists($mob))
				{
					$mob_obj = new ilObjMediaObject($mob);
					$mob_obj->exportFiles($a_export_dir);
					unset($mob_obj);
				}
			}
		}
	}
	
	/**
	 * Main export method for single object export AND export within an container (crs, cat, ...)
	 * @param $a_entity
	 * @param $a_schema_version
	 * @param $a_id
	 * @return string
	 */
	public function getXmlRepresentation($a_entity, $a_schema_version, $a_id)
	{
		$ref_id = current(ilObject::_getAllReferences($a_id));
		
		$date               = time();
		$this->export_dir   = $this->getAbsoluteExportDirectory();
		$this->subdir       = $date . "__" . IL_INST_ID . "__" . "xqsp" . "_" . $a_id;
		$this->filename     = $this->subdir . ".xml";
		$this->qti_filename ="xqsp_qti.xml";
		
		$this->plugin_object->includeClass('class.ilObjQuestionSetPool.php');
		
		$this->object     = new ilObjQuestionSetPool($ref_id);
		$this->xml_writer = new ilXmlWriter();
		
		$this->qpl_obj = new ilObjQuestionPool();
		// get Log File
		ilUtil::makeDirParents($this->getExportDirectory());
		$this->initLogFile();
		$this->exportPagesXML();
		
		return $this->xml_writer->xmlDumpMem();
	}
	
	/**
	 * @param $a_entity
	 * @return array
	 */
	public function getValidSchemaVersions($a_entity)
	{
		return array(
			'5.1.0' => array(
				'namespace'    => 'http://www.ilias.de/',
				'xsd_file'     => 'xqsp_5_1.xsd',
				'uses_dataset' => false,
				'min'          => '5.1.0',
				'max'          => '5.1.999'
			)
		);
	}
	
	/**
	 * @param $a_entity
	 * @param $a_target_release
	 * @param array $a_ids
	 * @return array
	 */
	public function getXmlExportHeadDependencies($a_entity, $a_target_release, $a_ids)
	{
		if("xqsp" == $a_entity)
		{
			// Array von QSP obj_ids
			$qpl_obj_ids = array();
			
			global $ilDB, $ilObjDataCache;
			//qs.id, qs.questionsetpool_fi, qst.question_fi, qpl_qst.obj_fi, 
			$res = $ilDB->query('
			SELECT qpl_qst.question_id, qpl_qst.original_id  
			FROM rep_robj_xqsp_qs qs
			INNER JOIN rep_robj_xqsp_qs_qst qst ON qs.id = qst.questionset_fi
			INNER JOIN qpl_questions qpl_qst ON qst.question_fi = qpl_qst.question_id
			WHERE ' . $ilDB->in('qs.questionsetpool_fi', $a_ids, false, 'integer'));
			
			$org_qst_ids = array();
			while($row = $ilDB->fetchAssoc($res))
			{
				$ids[] = $row;
				if($row['original_id'] !== NULL)
				{
					$org_qst_ids[] = $row['original_id'];
				}
				else
				{
					$org_qst_ids[] = $row['question_id'];
				}
			}
			
			if(count($org_qst_ids) > 0)
			{
				$res_2 = $ilDB->query('SELECT obj_fi FROM qpl_questions WHERE '. $ilDB->in('question_id',$org_qst_ids, false, 'integer'));
				
				while($row = $ilDB->fetchAssoc($res_2))
				{
					if($ilObjDataCache->lookupType($row['obj_fi']) == 'qpl')
					{
						$qpl_obj_ids[$row['obj_fi']] = $row['obj_fi'];
					}
				}
			}
			
			return array (array(
				"component" => "Modules/TestQuestionPool",
				"entity" => "qpl",
				"ids" => array_unique($qpl_obj_ids)));
		}
		
		return parent::getXmlExportHeadDependencies();		
	}
}
