<?php

require_once "abstract.ilPluginControllerTableGUI.php";

/**
 * Class ilQuestionSetPoolNodesGUI
 *
 * Date: 21.11.13
 * Time: 11:52
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolNodesGUI extends ilPluginControllerTableGUI
{

	public function __construct(ilObjQuestionSetPoolGUI $controller)
	{
		parent::__construct($controller);
		$this->addSubTabs();
	}

	/**
	 *
	 */
	public function editQuestion()
	{
		$this->controller->ensurePermission('write');

		if(!isset($_GET['q_id']) || !(int)$_GET['q_id'])
		{
			$this->listNodes();
			return;
		}

		$params = array(
			'set_id'  => (int)$_GET['set_id'],
			'path_id' => (int)$_GET['path_id'],
			'cmd'     => __CLASS__ . '.listNodes'
		);

		$question_id = (int)$_GET['q_id'];
		ilUtil::redirect('ilias.php?consumer_context=' . base64_encode(json_encode($params)) . '&baseClass=ilObjQuestionPoolGUI&ref_id=' . $this->controller->object->getRefId().'&cmd=editQuestionForTest&calling_consumer=' . $this->controller->object->getRefId() . '&q_id=' . $question_id);
	}

	public function listNodes()
	{
		$this->addToolbarButton("back_to_path_list", "ilQuestionSetPoolPathsGUI.listPaths",array(
				"set_id" => (int)$_GET["set_id"]
			));
		$this->addToolbarButton("add_node", "ilQuestionSetPoolNodeGUI.edit", array(
			"path_id" => (int)$_GET['path_id'],
			"set_id" =>(int) $_GET['set_id'],
		));

		$model = new ilQuestionSetPoolPath((int)$_GET['path_id']);
		$model->read();

		$this->ctrl->setParameter($this->controller, "set_id", (int)$_GET["set_id"]);
		$table = new ilQuestionSetPoolNodeTableGUI($this->controller, $this->plugin, (int)$_GET['path_id']);
		$table->setModelList($model->getNodes());
		$table->setTitle(
			  $model->getSet()->getTitle() . " - " .
			  $model->getTitle() . " - " .
			  $this->plugin->txt('nodes'));

		$table->addCommandButton("ilQuestionSetPoolNodesGUI.saveSorting", $this->plugin->txt("save_sorting"));
		$table->addCommandButton("ilQuestionSetPoolNodesGUI.saveDominantScoring", $this->plugin->txt("save_dominant_scoring"));

		$table->populate();

		$this->tpl->setContent($table->getHTML());
	}

	public function saveSorting()
	{
		$this->sortdata();
		ilUtil::sendSuccess($this->plugin->txt("saved_sorting"), true);
		$this->redirect("ilQuestionSetPoolNodesGUI.listNodes", array(
				"set_id" => (int)$_GET["set_id"],
				"path_id" => (int)$_GET["path_id"]
			));
	}

	public function saveDominantScoring()
	{
		$model = new ilQuestionSetPoolPath((int)$_GET["path_id"]);
		$model->read();
		$data = $model->getNodes()->getData();

		foreach($data as $element)
		{
			$node = new ilQuestionSetPoolNode($element["id"]);
			$node->read();
			if($_POST["dominant_scoring"][$node->getId()] === "")
			{
				$node->setDominantScoring(null);
			}
			else
			{
				$node->setDominantScoring((int)$_POST["dominant_scoring"][$node->getId()]);
			}
			$node->update();
		}

		ilUtil::sendSuccess($this->plugin->txt("saved_dominant_scoring"), true);
		$this->redirect("ilQuestionSetPoolNodesGUI.listNodes", array(
			"set_id" => $model->getSet()->getId(),
			"path_id" => $model->getId()
		));
	}

	protected function sortdata()
	{
		$model = new ilQuestionSetPoolPath((int)$_GET['path_id']);
		$model->read();
		$data = $model->getNodes()->getData();

		foreach($data as $key => $element)
		{
			if($element["node_index"] != 1)
			{
				$minimumTwo               = (int)$_POST["sorting"][$element["id"]];
				$data[$key]["node_index"] = $minimumTwo > 1 ? $minimumTwo : 2;
			}
		}

		usort($data, function($a,$b){
			return $a["node_index"] - $b["node_index"];
		});

		if(!$this->conditionsAreValid($data))
		{
			ilUtil::sendFailure($this->plugin->txt("sorting_error_jmp_conditions"), true);
			$this->redirect("ilQuestionSetPoolNodesGUI.listNodes", array(
					"set_id" => (int)$_GET["set_id"],
					"path_id" => (int)$_GET["path_id"]
				));
		}

		for($i = 0; $i < count($data); $i++)
		{
			$data[$i]["node_index"] = ($i + 1);
			$model->getNodes()->update($data[$i]);
		}

		return $data;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "content";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			"models/class.ilQuestionSetPoolPath.php",
			"tables/class.ilQuestionSetPoolNodeTableGUI.php",
			"utils/abstract.ilQuestionSetPoolConditionValidator.php"
		);
	}

	private function addSubTabs()
	{
		global $DIC;
		$ilTabs = $DIC->tabs();
		
		$this->ctrl->setParameter($this->controller, "set_id", (int)$_GET['set_id']);

		$ilTabs->addSubTab("set_questions", $this->plugin->txt("questions"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolSetQuestionsGUI.listQuestions"));

		$ilTabs->addSubTab("paths", $this->plugin->txt("paths"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.listPaths"));

		$ilTabs->addSubTab("path_overview", $this->plugin->txt("path_overview"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGraphGUI.showPathOverview"));

		$ilTabs->activateSubTab("paths");

		$this->ctrl->setParameter($this->controller, "set_id", "");
	}

	/**
	 * @param $data
	 *
	 * @return bool
	 */
	protected function conditionsAreValid($data)
	{
		for ($index = 1; $index < count($data); $index++)
		{
			$node = new ilQuestionSetPoolNode($data[$index]["id"]);
			$node->read();
			$node->setIndex($data[$index]["node_index"]);

			if(!ilQuestionSetPoolConditionValidator::isValid($node->getInitialCondition(),$node,$data,$this->plugin))
			{
				return false;
			}

			foreach ($data[$index]["jump_conditions"] as $jump_condition)
			{
				if(!ilQuestionSetPoolConditionValidator::isValid($jump_condition["jump_condition"],$node,$data,$this->plugin))
				{
					return false;
				}

				$error = false;
				for ($inner_index = 0; $inner_index <= $index; $inner_index++)
				{
					if ($data[$inner_index]["question_fi"] == $jump_condition["question_fi"] || $jump_condition["question_fi"] == null)
					{
						$error = false;
						break;
					}
					else
					{
						$error = true;
					}
				}
				if ($error)
				{
					return false;
				}
			}
		}
		return true;
	}
}
 