<?php
require_once 'abstract.ilPluginControllerGUI.php';
require_once dirname(__FILE__) . '/../../vendor/autoload.php';
/**
 * Class ilQuestionSetPoolPathGraphGUI
 */
class ilQuestionSetPoolPathGraphGUI extends  ilPluginControllerGUI
{
	/**
	 * @param ilObjQuestionSetPoolGUI $controller
	 */
	public function __construct(ilObjQuestionSetPoolGUI $controller)
	{
		parent::__construct($controller);
		$this->addSubTabs();
	}

	/**
	 *
	 */
	public function showPathOverview()
	{
		$set = new ilQuestionSetPoolSet((int)$_GET['set_id']);
		$set->read();

		$template = new ilTemplate('tpl.il_xqsp_path_overview.html', true, true, 'Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool');

		$this->renderPathTable($set, $template);
		$this->ctrl->setParameter($this->controller, 'set_id', (int)$_GET['set_id']);

		$template->setVariable('SET_TITLE', $set->getTitle());
		$template->setVariable('GRAPH_TITLE', $this->plugin->txt("graph_viz_title"));
		$template->setVariable('PATH_GRAPH_SRC', $this->renderPathGraph());

		$this->tpl->setContent($template->get());
	}

	/**
	 *
	 */
	public function renderPathGraph()
	{
		$set = new ilQuestionSetPoolSet((int)$_GET['set_id']);
		$set->read();

		$graph = new Fhaculty\Graph\Graph();

		$pathes = $set->getPathList()->getData();
		foreach($pathes as $path)
		{
			$model = new ilQuestionSetPoolPath($path['id']);
			$model->read();
			$nodes = $model->getNodes()->getData();
			$num_nodes = count($nodes);
			for($i = 0; $i < $num_nodes; $i++)
			{
				$nodeId = 'Q' . $nodes[$i]['question_index'];
				try {
					$graph->getVertex($nodeId);
				} catch (Exception $exception) {
					$vertex = $graph->createVertex($nodeId);
					$vertex->setAttribute('graphviz.shape', 'box');
				}
			}
		}
		$vertex = $graph->createVertex($this->controller->plugin->txt('end_of_set'));
		$vertex->setAttribute('graphviz.shape', 'box');

		$edges = array();

		foreach($pathes as $path)
		{
			$model = new ilQuestionSetPoolPath($path['id']);
			$model->read();
			$nodes = $model->getNodes()->getData();
			$num_nodes = count($nodes);
			for($i = 0; $i < $num_nodes; $i++)
			{
				if(isset($nodes[$i + 1]))
				{
					$edges["Q".$nodes[$i]['question_index']]["Q".$nodes[$i + 1]['question_index']]['initial'][trim($nodes[$i]['initial_condition'])] = $nodes[$i]['initial_condition'];
				}
				else
				{
					$edges["Q".$nodes[$i]['question_index']][$this->controller->plugin->txt('end_of_set')]['initial'][trim($nodes[$i]['initial_condition'])] = $nodes[$i]['initial_condition'];
				}

				foreach((array)$nodes[$i]['jump_conditions'] as $condition)
				{
					if($condition['question_index'])
					{
						$edges["Q".$nodes[$i]['question_index']]["Q".$condition['question_index']]['jump'][trim($condition['jump_condition'])] = $condition['jump_condition'];
					}
					else
					{
						$edges["Q".$nodes[$i]['question_index']][$this->controller->plugin->txt('end_of_set')]['end'][trim($condition['jump_condition'])] = $condition['jump_condition'];
					}
				}
			}
		}

		foreach($edges as $node_source => $data_target)
		{
			foreach($data_target as $node_target => $data_conditions)
			{
				if(count($data_conditions['initial']) > 0)
				{
					$initial = join("\n", array_filter($data_conditions['initial']));
					$edge = new Fhaculty\Graph\Edge\Directed($graph->getVertex($node_source), $graph->getVertex($node_target));
					$edge->setAttribute('graphviz.color', 'darkgreen');
					$edge->setAttribute('graphviz.label', $initial);
				}
				if(count($data_conditions['jump']) > 0)
				{
					$jump = join("\n", array_filter($data_conditions['jump']));
					$edge = new Fhaculty\Graph\Edge\Directed($graph->getVertex($node_source), $graph->getVertex($node_target));
					$edge->setAttribute('graphviz.color', 'blue');
					$edge->setAttribute('graphviz.label', $jump);
				}
				if(count($data_conditions['end']) > 0)
				{
					$end = join("\n", array_filter($data_conditions['end']));
					$edge = new Fhaculty\Graph\Edge\Directed($graph->getVertex($node_source), $graph->getVertex($node_target));
					$edge->setAttribute('graphviz.color', 'red');
					$edge->setAttribute('graphviz.label', $end);
				}
			}
		}

		$graphviz = new Graphp\GraphViz\GraphViz();
		$imageSrc =  $graphviz->createImageSrc($graph);
		return $imageSrc;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return 'content';
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			'models/class.ilQuestionSetPoolSet.php',
			'models/class.ilQuestionSetPoolPath.php'
		);
	}

	private function addSubTabs()
	{
		global $DIC;
		$ilTabs = $DIC->tabs();

		$this->ctrl->setParameter($this->controller, "set_id", (int)$_GET['set_id']);

		$ilTabs->addSubTab("set_questions", $this->plugin->txt("questions"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolSetQuestionsGUI.listQuestions"));

		$ilTabs->addSubTab("paths", $this->plugin->txt("paths"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.listPaths"));

		$ilTabs->addSubTab("path_overview", $this->plugin->txt("path_overview"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGraphGUI.showPathOverview"));

		$ilTabs->activateSubTab("path_overview");

		$this->ctrl->setParameter($this->controller, "set_id", "");
	}

	/**
	 * @param $set
	 * @param $template
	 */
	protected function renderPathTable($set, $template)
	{
		$pathes = $set->getPathList()->getData();
		foreach ($pathes as $path_data)
		{
			$path = new ilQuestionSetPoolPath($path_data['id']);
			$path->read();
			$nodes = $path->getNodes()->getData();

			foreach ($nodes as $node)
			{
				$template->setCurrentBlock("initial_condition");
				$condition = $node["initial_condition"];
				if ($condition == "")
				{
					$condition = $this->plugin->txt("empty_condition");
				}
				$template->setVariable("INITIAL_CONDITION", $condition);
				$template->parseCurrentBlock();

				$template->setCurrentBlock("question");
				$template->setVariable("QUESTION_INDEX", $node["question_index"]);
				$template->parseCurrentBlock();

				if (count($node["jump_conditions"]) > 0)
				{
					foreach ($node["jump_conditions"] as $jump_condition)
					{
						$condition = "";

						if ($jump_condition["question_index"] != null)
						{
							$condition .= $this->plugin->txt("target_question") . ": ";
							$condition .= "Q" . $jump_condition["question_index"];
						}
						else
						{
							$condition .= $this->plugin->txt("tst_termination");
						}
						$template->setCurrentBlock("jump_condition");
						$template->setVariable("JUMP_CONDITION", $jump_condition["jump_condition"]);
						$template->setVariable("TARGET", $condition . "<br />". $this->plugin->txt("repititions") . ": " . $jump_condition["repititions"]);
						$template->parseCurrentBlock();
					}
				}
				else
				{
					$template->setCurrentBlock("jump_condition");
					$template->setVariable("JUMP_CONDITION", "&nbsp;");
					$template->setVariable("TARGET", "&nbsp;");
					$template->parseCurrentBlock();
				}
				$template->setCurrentBlock("jump_conditions");
				$template->parseCurrentBlock();
			}

			$template->setCurrentBlock("test_end");
			$template->setVariable("END_TEST", $this->plugin->txt('end_of_test'));
			$template->parseCurrentBlock();

			$template->setCurrentBlock("path");
			$template->setVariable("PATH_TITLE", $path->getTitle());
			$template->parseCurrentBlock();
		}
	}
}
