<?php

require_once "abstract.ilPluginModelList.php";

/**
 * Class ilQuestionSetPoolNodeList
 *
 * Date: 21.11.13
 * Time: 12:21
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolNodeList extends ilPluginModelList
{

	/**
	 * @var ilQuestionSetPoolPath
	 */
	protected $path;

	public function __construct($path)
	{
		$this->path = $path;

		parent::__construct();
	}

	/**
	 * @param array $params
	 * @param array $filter
	 *
	 * @return array
	 */
	public function loadData($params = array(), $filter = array())
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilQuestionSetPoolNodeProvider.php";

		$provider = new ilQuestionSetPoolNodeProvider();
		$filter["path_fi"] = $this->path->getId();
		return $provider->loadData($filter, $params);
	}

	public function update($data)
	{
		return $this->db->update(
			"rep_robj_xqsp_node",
			array(
				"node_index" => array("integer", $data["node_index"])
			),
			array(
				"id" => array("integer", $data["id"])
			)
		);
	}

	public function getQuestionIds()
	{

	}

	/**
	 * @param ilQuestionSetPoolNode $current_node
	 *
	 * @return ilQuestionSetPoolNode
	 */
	public function getNextNode($current_node)
	{
		$data = $this->getData();

		foreach($data as $item)
		{
			if($item["node_index"] > $current_node->getIndex())
			{
				$node = new ilQuestionSetPoolNode($item["id"]);
				$node->read();
				return $node;
			}
		}

		return null;
	}

	/**
	 * @param ilQuestionSetPoolNode $current_node
	 *
	 * @return IlQuestionSetPoolNode|null
	 */
	public function getPreviousNode($current_node)
	{
		$data = $this->getData();

		$prev = null;
		foreach($data as $item)
		{
			if($item["node_index"] == $current_node->getIndex() && $prev != null)
			{
				$prev = new ilQuestionSetPoolNode($prev["id"]);
				$prev->read();
				return $prev;
			}
			$prev = $item;
		}
		return null;
	}

	/**
	 * @param int $question_id
	 *
	 * @return ilQuestionSetPoolNode|null
	 */
	public function getNodeByQuestionId($question_id)
	{
		$data = $this->getData();

		foreach($data as $item)
		{
			if($item["question_fi"] == $question_id)
			{
				$node = new ilQuestionSetPoolNode($item["id"]);
				$node->read();
				return $node;
			}
		}
		return null;
	}

	/**
	 * @param array $data
	 *
	 * @return array
	 */
	public function createObjectsFromData($data)
	{
		require_once "class.ilQuestionSetPoolJumpConditionList.php";
		require_once "class.ilQuestionSetPoolNode.php";
		foreach($data as $key => $element)
		{
			$node = new ilQuestionSetPoolNode($element["id"]);
			$node->read();

			$list = new ilQuestionSetPoolJumpConditionList($node);
			$data[$key]["jump_conditions"] = $list->getData();
		}

		return $data;
	}
}
 