<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/abstract.ilPluginModelList.php";

/**
 * Class ilQuestionSetPoolSetQuestionList
 *
 * Date: 26.09.13
 * Time: 12:21
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolSetQuestionList extends ilPluginModelList
{

	/**
	 * @var ilQuestionSetPoolSet
	 */
	protected $set;

	public function __construct($set)
	{
		$this->set = $set;

		parent::__construct();
	}

	/**
	 * @param array $params
	 * @param array $filter
	 *
	 * @return array
	 */
	public function loadData($params = array(), $filter = array())
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilQuestionSetPoolSetQuestionsProvider.php";

		$provider = new ilQuestionSetPoolSetQuestionsProvider();
		$filter["questionset_fi"] = $this->set->getId();
		return $provider->loadData($filter, $params);
	}

	/**
	 * @param array $question_ids
	 * @param ilObjQuestionSetPool $question_set_pool
	 *
	 * @return bool
	 */
	public function insert($question_ids, $question_set_pool)
	{

		require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";

		$question_titles = $this->getQuestionTitles();
		
		foreach($question_ids as $question_id)
		{
			$question = assQuestion::_instantiateQuestion($question_id);
			$obj_id = $question->getObjId();

			$title = $question->getTitle();
			if(in_array($title, $question_titles))
			{
				$i = 2;
				while(in_array($title . ' (' . $i . ')', $question_titles))
				{
					$i++;
				}
				$title .= ' (' . $i . ')';
			}
			$question_titles[] = $title;

			//$copy_id = $question->copyObject($this->question_set_pool->getId());
			$copy_id = $question->duplicate(true, $title,"","",$question_set_pool->getId());

			if($question->getObjId() == $obj_id)
			{
				$this->db->update(
					"qpl_questions",
					array(
						"original_id" => array("integer", null)
					),
					array(
						"question_id" => array("integer", $copy_id)
					)
				);
			}


			$success = $this->db->queryF(
				"INSERT INTO rep_robj_xqsp_qs_qst (questionset_fi, question_fi, question_index)
				(SELECT %s,%s,
				CASE WHEN MAX(question_index) IS NULL
					THEN 1
					ELSE MAX(question_index) + 1
				END
				FROM rep_robj_xqsp_qs_qst WHERE questionset_fi = %s)",
				array("integer", "integer", "integer"),
				array($this->set->getId(), $copy_id, $this->set->getId())
			);

			if(!$success)
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * @return array
	 */
	public function getQuestionTitles()
	{
		global $DIC;
		$ilDB = $DIC->database();

		$titles = array();
		$result = $ilDB->queryF("SELECT DISTINCT(qpl_questions.title) FROM rep_robj_xqsp_qs_qst, qpl_questions WHERE rep_robj_xqsp_qs_qst.questionset_fi = %s AND rep_robj_xqsp_qs_qst.question_fi = qpl_questions.question_id ORDER BY rep_robj_xqsp_qs_qst.question_index",
			array('integer'),
			array($this->set->getId())
		);

		while ($row = $ilDB->fetchAssoc($result))
		{
			$titles[] = $row['title'];
		}

		return $titles;
	}

	/**
	 * @param array $question_ids
	 * @param ilQuestionSetPoolSet $old_set
	 * @param integer $container_obj_id Container id (e.g. question set pool obj_id or forma test obj_id)
	 *
	 * @return array
	 */
	public function duplicate($question_ids, $old_set, $container_obj_id)
	{
		require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";
		$new_question_ids = array();
		foreach($question_ids as $question_id)
		{
			$question = assQuestion::_instantiateQuestion($question_id);
			$original_obj_fi = $question->getObjId();
			$copy_id = $question->duplicate(true, "","","", $container_obj_id);

			$new_question_ids[$question_id] = $copy_id;

			if($original_obj_fi != $container_obj_id)
			{
				$this->db->manipulateF(
					'UPDATE qpl_questions SET original_id = %s WHERE question_id = %s',
					array('integer', 'integer'),
					array($question_id, $copy_id)
				);
			}

			$success = $this->db->queryF(
				"INSERT INTO rep_robj_xqsp_qs_qst (questionset_fi, question_fi, question_index)
				(SELECT %s, %s, question_index
				FROM rep_robj_xqsp_qs_qst WHERE questionset_fi = %s AND question_fi = %s
				)",
				array("integer", "integer", "integer", "integer"),
				array($this->set->getId(), $copy_id, $old_set->getId(), $question_id)
			);
			if(!$success)
			{
				return array();
			}
		}
		return $new_question_ids;
	}

	/**
	 * @param array $question_ids
	 * @param ilQuestionSetPoolSet $old_set
	 * @param int $container_obj_id
	 * @param array $old_questions
	 */
	public function synchronize($question_ids, $old_set, $container_obj_id, $old_questions)
	{
		require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";
		$new_question_ids = array();
		foreach($question_ids as $question_id)
		{
			$question = assQuestion::_instantiateQuestion($question_id);
			$old_original_id = $question->getOriginalId();
			$copy_id = $question->duplicate(true, "","","", $container_obj_id);

			$new_question_ids[$question_id] = $copy_id;

			// Change original id of base question (Testquestion)
			$this->db->manipulateF(
				"UPDATE qpl_questions SET original_id = %s WHERE original_id = %s",
				array("integer", "integer"),
				array($copy_id, $old_original_id)
			);

			foreach($old_questions as $old_question)
			{
				if($old_question["question_id"] == $old_original_id)
				{
					$old_original_id = $old_question["original_id"];
				}
			}

			$this->db->manipulateF(
				"UPDATE qpl_questions SET original_id = %s WHERE question_id = %s",
				array("integer", "integer"),
				array($old_original_id, $copy_id)
			);

			$success = $this->db->queryF(
				"INSERT INTO rep_robj_xqsp_qs_qst (questionset_fi, question_fi, question_index)
				(SELECT %s, %s, question_index
				FROM rep_robj_xqsp_qs_qst WHERE questionset_fi = %s AND question_fi = %s
				)",
				array("integer", "integer", "integer", "integer"),
				array($this->set->getId(), $copy_id, $old_set->getId(), $question_id)
			);
			if(!$success)
			{
				return array();
			}
		}

		return $new_question_ids;
	}

	/**
	 * @param array $question_ids
	 *
	 * @return bool
	 */
	public function remove($question_ids)
	{
		foreach($question_ids as $question_id)
		{
			$success = $this->db->manipulateF(
				"DELETE FROM rep_robj_xqsp_qs_qst WHERE question_fi = %s",
				array("integer"),
				array($question_id)
			);

			require_once 'Modules/TestQuestionPool/classes/class.assQuestion.php';
			/**
			 * @var $gui assQuestionGUI
			 */
			$question = assQuestion::_instantiateQuestion($question_id);
			if($question instanceof assQuestion)
			{
				/**
				 * @var $object assQuestion
				 */
				$question->delete($question_id);
				//@todo überprüfen ob Löschen von Verzeichnissen bei Fragen mit Dateiupload (zB: Imagemap + vertikale Anordnungsfrage) funktioniert 
				
			}

			if(!$success)
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * @param array $data
	 *
	 * @return array
	 */
	public function createObjectsFromData($data)
	{
		return $data;
	}
	
	/**
	 * @param nt $question_id
	 * @param null $original_question_id
	 * @return bool||int $copy_id
	 */
	public function import($question_id, $original_question_id = NULL)
	{
		require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";
		
		$question_titles = $this->getQuestionTitles();
		
		$question = assQuestion::_instantiateQuestion($question_id);
		$obj_id = $question->getObjId();
		
		$title = $question->getTitle();
		if(in_array($title, $question_titles))
		{
			$i = 2;
			while(in_array($title . ' (' . $i . ')', $question_titles))
			{
				$i++;
			}
			$title .= ' (' . $i . ')';
		}
		$question_titles[] = $title;
		
		if($question->getObjId() == $obj_id)
		{
			if($original_question_id == 0)
			{
				$original_question_id = NULL;
			}
			
			$this->db->update(
					"qpl_questions",
					array(
							"original_id" => array("integer", $original_question_id)
					),
					array(
							"question_id" => array("integer", $question_id)
					)
			);
		}
		
		$success = $this->db->queryF(
				"INSERT INTO rep_robj_xqsp_qs_qst (questionset_fi, question_fi, question_index)
			(SELECT %s,%s,
			CASE WHEN MAX(question_index) IS NULL
				THEN 1
				ELSE MAX(question_index) + 1
			END
			FROM rep_robj_xqsp_qs_qst WHERE questionset_fi = %s)",
				array("integer", "integer", "integer"),
				array($this->set->getId(), $question_id, $this->set->getId())
		);
	
		if(!$success)
		{
			return false;
		}
		return $question_id;
	}
	
	/**
	 * @param  array               $question_ids
	 * @param ilQuestionSetPoolSet $source_set
	 * @param ilQuestionSetPoolSet $target_set
	 * @return array
	 */
	public function cloneQuestionList($question_ids, ilQuestionSetPoolSet $source_set, ilQuestionSetPoolSet $target_set)
	{
		require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";
		$new_question_ids = array();
		foreach($question_ids as $question_id)
		{
			$question = assQuestion::_instantiateQuestion($question_id);
			$copy_id = $question->duplicate(true, "","","", $target_set->getContainerObjId());
			
			$original_obj_fi = $question->getObjId();

			$new_question_ids[$question_id] = $copy_id;

			if($original_obj_fi != $target_set->getContainerObjId())
			{
				$this->db->manipulateF(
						'UPDATE qpl_questions SET original_id = %s WHERE question_id = %s',
						array('integer', 'integer'),
						array($question_id, $copy_id)
				);
			}
			
			$this->db->queryF(
					"INSERT INTO rep_robj_xqsp_qs_qst (questionset_fi, question_fi, question_index)
				(SELECT %s, %s, question_index
				FROM rep_robj_xqsp_qs_qst WHERE questionset_fi = %s AND question_fi = %s
				)",
					array("integer", "integer", "integer", "integer"),
					array($target_set->getId(), $copy_id, $source_set->getId(), $question_id)
			);
		}
		return $new_question_ids;
	}
}
